/*
 * Copyright (C) 2020 ~ 2021 UnionTech Software Technology Co., Ltd.
 *
 * Author:     sunkang <sunkang@uniontech.com>
 *
 * Maintainer: sunkang <sunkang@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ASYNC_HTTPCLIENTAPI_H
#define ASYNC_HTTPCLIENTAPI_H

#include <QObject>
#include <QTimer>
#include <QFile>
#include <QNetworkRequest>
#include <QNetworkReply>

#include <functional>

class AsyncHttpClientApiPrivate;
class AsyncHttpClientApi : public QObject
{
    Q_OBJECT
public:
    typedef std::function<void(QString, int)> GetRequestCallback; // http GET请求回调函数
    typedef std::function<void(QString, int)> PostRequestCallback; // http POST请求回调函数
    typedef std::function<void(QString, int)> PutRequestCallback; // http PUT请求回调函数
    // http 请求回调函数,这个函数用于添加其他请求头信息
    typedef std::function<void(QString, int, QList<QNetworkReply::RawHeaderPair>)> HttpRequestCallback;
    // 下载文件的回调函数，函数第一个参数为文件名，第二个参数为返回码
    typedef std::function<void(QString, int)> DownloadCallback;

    ~AsyncHttpClientApi();
    static AsyncHttpClientApi *getInstance();

    void setSSLConfig(const QString &strPath, const QString &strLocalKey, const QString &strPrivateKey);

    // http GET 请求,支持多线程调用
    void getRequest(const GetRequestCallback &callback, const QString &strUrl,
                    const QString &strBody = QString(), int timeoutSeconds = 60);

    // http POST 请求,支持多线程调用
    void postRequest(const PostRequestCallback &callback, const QString &strUrl,
                     const QString &strBody = QString(), int timeoutSeconds = 60);

    // http POST 请求,支持多线程调用(不带回调函数)
    void postRequest(const QString &strUrl, const QString &strBody = QString(), int timeoutSeconds = 60);

    // http PUT 请求,支持多线程调用
    void putRequest(const PutRequestCallback &callback, const QString &strUrl,
                    const QString &strBody = QString(), int timeoutSeconds = 60);

    // 自定义http请求,支持多线程调用
    void httpRequest(const HttpRequestCallback &callback,
                     const QString &strUrl,
                     const QByteArray &verb,
                     const QByteArray &strBody,
                     const QList<QNetworkReply::RawHeaderPair> &rawHeaders,
                     int timeoutSeconds = 60);

    // 异步下载文件,文件下载完成或者超时会调用回调函数, 支持多线程调用
    void downloadFile(const DownloadCallback &callback,
                      const QString &fileName,
                      const QString &strUrl,
                      const QString &strBody = QString(),
                      bool supportBreakPoint = false,
                      const QList<QNetworkReply::RawHeaderPair> &rawHeaders = QList<QNetworkReply::RawHeaderPair>(),
                      int timeoutSeconds = 3600);

private:
    typedef std::function<void()> TaskCallback; // 任务回调类型(内部使用)

    // 判断是否在初始化线程
    bool isInitThread() const;
    // http get请求的实际执行函数
    void getRequestInLoop(const GetRequestCallback &callback, const QString &strUrl,
                          const QString &strBody, int timeoutSeconds);
    // http post请求的实际执行函数
    void postRequestInLoop(const PostRequestCallback &callback, const QString &strUrl,
                           const QString &strBody, int timeoutSeconds);
    // http put请求的实际执行函数
    void putRequestInLoop(const PutRequestCallback &callback, const QString &strUrl,
                          const QString &strBody, int timeoutSeconds);
    // http 请求的实际执行函数
    void httpRequestInLoop(const HttpRequestCallback &callback,
                           const QString &strUrl,
                           const QByteArray &verb,
                           const QByteArray &strBody,
                           const QList<QNetworkReply::RawHeaderPair> &rawHeaders,
                           int timeoutSeconds);
    // 实际执行异步下载文件的操作
    void downloadFileInLoop(const DownloadCallback &callback,
                            const QString &fileName,
                            const QString &strUrl,
                            const QString &strBody,
                            bool supportBreakPoint,
                            const QList<QNetworkReply::RawHeaderPair> &rawHeaders,
                            int timeoutSeconds);

    void getRequestProcess(GetRequestCallback callback, QTimer *pTimer, QNetworkReply *pReply);
    void postRequestProcess(PostRequestCallback callback, QTimer *pTimer, QNetworkReply *pReply);
    void putRequestProcess(PutRequestCallback callback, QTimer *pTimer, QNetworkReply *pReply);
    void httpRequestProcess(HttpRequestCallback callback, QTimer *pTimer, QNetworkReply *pReply);
    void downloadProcess(DownloadCallback callback,
                         QTimer *pTimer,
                         QNetworkReply *pReply,
                         QFile *pFile,
                         qint64 bytesReceived,
                         qint64 bytesTotal);

    // http GET 请求超时处理
    void onGetTimeout(GetRequestCallback callback, QTimer *pTimer, QNetworkReply *pReply);
    // http POST 请求超时处理
    void onPostTimeout(PostRequestCallback callback, QTimer *pTimer, QNetworkReply *pReply);
    // http PUT 请求超时处理
    void onPutTimeout(PutRequestCallback callback, QTimer *pTimer, QNetworkReply *pReply);
    // 自定义http 请求的超时处理
    void onHttpTimeout(HttpRequestCallback callback, QTimer *pTimer, QNetworkReply *pReply);
    // 下载文件超时处理
    void onDownloadTimeout(DownloadCallback callback, QFile *pFile, QTimer *pTimer, QNetworkReply *pReply);

Q_SIGNALS:
    void runTaskSignal(TaskCallback task); // 内部使用信号

private Q_SLOTS:
    void onRunTask(TaskCallback task);

private:
    explicit AsyncHttpClientApi(QObject *parent = nullptr);

    const QScopedPointer<AsyncHttpClientApiPrivate> dPtr;
    Q_DECLARE_PRIVATE_D(dPtr, AsyncHttpClientApi)
    void setPeerVerifyMode(QString url);
};

#endif // ASYNC_HTTPCLIENTAPI_H
