/*
 * Copyright (C) 2020 ~ 2021 UnionTech Software Technology Co., Ltd.
 *
 * Author:     wangmingjun <wangmingjun@uniontech.com>
 *
 * Maintainer: wangmingjun <wangmingjun@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef QSYSLOG_H
#define QSYSLOG_H

#include <syslog.h>
#include <QtGlobal>
#include <QString>
#include <QByteArray>
#include <QCoreApplication>
#include <QFileInfo>
#include <QDateTime>
#include <QLoggingCategory>
#include <QStringList>
#include <qlogging.h>

namespace udcp {

/**
 * @brief The level enum
 * 日志等级划分,参考syslog头文件,与syslog头文件中的宏定义等级相同，切勿随意改动
 */
enum LEVEL {
    uLOG_EMERG = 0, /* system is unusable */
    uLOG_ALERT = 1, /* action must be taken immediately */
    uLOG_CRIT = 2, /* critical conditions */
    uLOG_ERR = 3, /* error conditions */
    uLOG_WARNING = 4, /* warning conditions */
    uLOG_NOTICE = 5, /* normal but significant condition */
    uLOG_INFO = 6, /* informational */
    uLOG_DEBUG = 7, /* debug-level messages */
};

/**
 * @brief logLevel
 * 存储日志打印等级，禁止手动调用此变量，由openlog初始化，默认日志等级为uLOG_INFO
 */
static int logLevel = LEVEL::uLOG_INFO;

/**
 * @brief uSyslogMessageHandler
 * 实际日志处理函数，供qInstallMessageHandler注册使用，无需手动调用次函数
 *
 * @param type
 * 日志类型 QtDebugMsg, QtWarningMsg, QtCriticalMsg, QtFatalMsg, QtInfoMsg
 * 如果日志类型是QtInfoMag则将日志内容输出完毕后会打印当前程序运行堆栈内容后程序core-dump
 *
 * @param context
 * 调用日志时的上下文，可通过此变量访问调用日志函数时的环境，如行号，函数名等
 *
 * @param msg
 * 具体日志内容
 */
void uSyslogMessageHandler(QtMsgType type, const QMessageLogContext &context, const QString &msg)
{
    int level;
    QString logtype;
    switch (type) {
    case QtDebugMsg:
        level = LOG_DEBUG;
        logtype = "debug";
        break;
    case QtInfoMsg:
        level = LOG_INFO;
        logtype = "info";
        break;
    case QtWarningMsg:
        level = LOG_WARNING;
        logtype = "warning";
        break;
    case QtCriticalMsg:
        level = LOG_CRIT;
        logtype = "critical";
        break;
    case QtFatalMsg:
        level = LOG_ERR;
        logtype = "fatal";
        break;
    }
    if (!msg.isEmpty() && (level <= logLevel)) {
        QString strTime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss.zzz");
        QString filePath = QString(context.file);
        QStringList list = filePath.split("/");
        syslog(level, "%s %s [ %s %s %u]  %s \n",
               strTime.toStdString().data(),
               logtype.toStdString().data(),
               list.last().toUtf8().data(),
               context.function,
               context.line,
               msg.toLocal8Bit().constData());
    }
}

/**
 * @brief uOpenlog
 * 打开日志系统,APP_NAME是程序在日志系统中需要显示的名称，未定义则使用当前运行程序名称作为日志系统中程序名称。
 *
 * @param lv
 * level类型，日志等级
 */
void uOpenlog(LEVEL lv = LEVEL::uLOG_INFO)
{
#ifdef APP_NAME
    const char *app_name = APP_NAME;
#else
    const char *app_name = QFileInfo(QCoreApplication::applicationFilePath()).fileName().toLatin1().data();
#endif

    if (LEVEL::uLOG_DEBUG == lv) {
        QLoggingCategory::defaultCategory()->setEnabled(QtDebugMsg, true);
    } else {
        QLoggingCategory::defaultCategory()->setEnabled(QtDebugMsg, false);
    }

    logLevel = lv;

    openlog(app_name, (LOG_CONS | LOG_PERROR | LOG_PID), LOG_DAEMON);
    qInstallMessageHandler(uSyslogMessageHandler);
}

/**
 * @brief uCloselog
 * 关闭日志系统（一般情况无需手动调用次函数关闭日志系统）
 */
void uCloselog()
{
    closelog();
}

} // namespace udcp

#endif // QSYSLOG_H
